from __future__ import annotations

from setuptools import setup

setup(
    name="testrepo",
    version="0.1",
    packages=["mypackage"],
    description="A test package with commas in file names",
    include_package_data=True,
    package_data={"mypackage.data": ["*"]},
)



from __future__ import annotations

from setuptools import setup

setup(
    name="complex-dist",
    version="0.1",
    description="Another testing distribution \N{SNOWMAN}",
    long_description="Another testing distribution \N{SNOWMAN}",
    author="Illustrious Author",
    author_email="illustrious@example.org",
    url="http://example.org/exemplary",
    packages=["complexdist"],
    setup_requires=["wheel", "setuptools"],
    install_requires=["quux", "splort"],
    extras_require={"simple": ["simple.dist"]},
    tests_require=["foo", "bar>=10.0.0"],
    entry_points={
        "console_scripts": [
            "complex-dist=complexdist:main",
            "complex-dist2=complexdist:main",
        ],
    },
)



from __future__ import annotations

from setuptools import setup

setup(
    name="unicode.dist",
    version="0.1",
    description="A testing distribution \N{SNOWMAN}",
    packages=["unicodedist"],
)



from __future__ import annotations

import os.path
import shutil
import stat
import subprocess
import sys
from zipfile import ZipFile

import pytest

from wheel.bdist_wheel import bdist_wheel
from wheel.wheelfile import WheelFile

DEFAULT_FILES = {
    "dummy_dist-1.0.dist-info/top_level.txt",
    "dummy_dist-1.0.dist-info/METADATA",
    "dummy_dist-1.0.dist-info/WHEEL",
    "dummy_dist-1.0.dist-info/RECORD",
}
DEFAULT_LICENSE_FILES = {
    "LICENSE",
    "LICENSE.txt",
    "LICENCE",
    "LICENCE.txt",
    "COPYING",
    "COPYING.md",
    "NOTICE",
    "NOTICE.rst",
    "AUTHORS",
    "AUTHORS.txt",
}
OTHER_IGNORED_FILES = {
    "LICENSE~",
    "AUTHORS~",
}
SETUPPY_EXAMPLE = """\
from setuptools import setup

setup(
    name='dummy_dist',
    version='1.0',
)
"""


@pytest.fixture
def dummy_dist(tmpdir_factory):
    basedir = tmpdir_factory.mktemp("dummy_dist")
    basedir.join("setup.py").write(SETUPPY_EXAMPLE)
    for fname in DEFAULT_LICENSE_FILES | OTHER_IGNORED_FILES:
        basedir.join(fname).write("")

    basedir.join("licenses").mkdir().join("DUMMYFILE").write("")
    return basedir


def test_no_scripts(wheel_paths):
    """Make sure entry point scripts are not generated."""
    path = next(path for path in wheel_paths if "complex_dist" in path)
    for entry in ZipFile(path).infolist():
        assert ".data/scripts/" not in entry.filename


def test_unicode_record(wheel_paths):
    path = next(path for path in wheel_paths if "unicode.dist" in path)
    with ZipFile(path) as zf:
        record = zf.read("unicode.dist-0.1.dist-info/RECORD")

    assert "åäö_日本語.py".encode() in record


def test_licenses_default(dummy_dist, monkeypatch, tmpdir):
    monkeypatch.chdir(dummy_dist)
    subprocess.check_call(
        [sys.executable, "setup.py", "bdist_wheel", "-b", str(tmpdir), "--universal"]
    )
    with WheelFile("dist/dummy_dist-1.0-py2.py3-none-any.whl") as wf:
        license_files = {
            "dummy_dist-1.0.dist-info/" + fname for fname in DEFAULT_LICENSE_FILES
        }
        assert set(wf.namelist()) == DEFAULT_FILES | license_files


@pytest.mark.parametrize(
    "config_file, config",
    [
        ("setup.cfg", "[metadata]\nlicense_files=licenses/*\n  LICENSE"),
        ("setup.cfg", "[metadata]\nlicense_files=licenses/*, LICENSE"),
        (
            "setup.py",
            SETUPPY_EXAMPLE.replace(
                ")", "  license_files=['licenses/DUMMYFILE', 'LICENSE'])"
            ),
        ),
    ],
)
def test_licenses_override(dummy_dist, monkeypatch, tmpdir, config_file, config):
    dummy_dist.join(config_file).write(config)
    monkeypatch.chdir(dummy_dist)
    subprocess.check_call(
        [sys.executable, "setup.py", "bdist_wheel", "-b", str(tmpdir), "--universal"]
    )
    with WheelFile("dist/dummy_dist-1.0-py2.py3-none-any.whl") as wf:
        license_files = {
            "dummy_dist-1.0.dist-info/" + fname for fname in {"DUMMYFILE", "LICENSE"}
        }
        assert set(wf.namelist()) == DEFAULT_FILES | license_files


def test_licenses_disabled(dummy_dist, monkeypatch, tmpdir):
    dummy_dist.join("setup.cfg").write("[metadata]\nlicense_files=\n")
    monkeypatch.chdir(dummy_dist)
    subprocess.check_call(
        [sys.executable, "setup.py", "bdist_wheel", "-b", str(tmpdir), "--universal"]
    )
    with WheelFile("dist/dummy_dist-1.0-py2.py3-none-any.whl") as wf:
        assert set(wf.namelist()) == DEFAULT_FILES


def test_build_number(dummy_dist, monkeypatch, tmpdir):
    monkeypatch.chdir(dummy_dist)
    subprocess.check_call(
        [
            sys.executable,
            "setup.py",
            "bdist_wheel",
            "-b",
            str(tmpdir),
            "--universal",
            "--build-number=2",
        ]
    )
    with WheelFile("dist/dummy_dist-1.0-2-py2.py3-none-any.whl") as wf:
        filenames = set(wf.namelist())
        assert "dummy_dist-1.0.dist-info/RECORD" in filenames
        assert "dummy_dist-1.0.dist-info/METADATA" in filenames


def test_limited_abi(monkeypatch, tmpdir):
    """Test that building a binary wheel with the limited ABI works."""
    this_dir = os.path.dirname(__file__)
    source_dir = os.path.join(this_dir, "testdata", "extension.dist")
    build_dir = tmpdir.join("build")
    dist_dir = tmpdir.join("dist")
    monkeypatch.chdir(source_dir)
    subprocess.check_call(
        [
            sys.executable,
            "setup.py",
            "bdist_wheel",
            "-b",
            str(build_dir),
            "-d",
            str(dist_dir),
        ]
    )


def test_build_from_readonly_tree(dummy_dist, monkeypatch, tmpdir):
    basedir = str(tmpdir.join("dummy"))
    shutil.copytree(str(dummy_dist), basedir)
    monkeypatch.chdir(basedir)

    # Make the tree read-only
    for root, _dirs, files in os.walk(basedir):
        for fname in files:
            os.chmod(os.path.join(root, fname), stat.S_IREAD)

    subprocess.check_call([sys.executable, "setup.py", "bdist_wheel"])


@pytest.mark.parametrize(
    "option, compress_type",
    list(bdist_wheel.supported_compressions.items()),
    ids=list(bdist_wheel.supported_compressions),
)
def test_compression(dummy_dist, monkeypatch, tmpdir, option, compress_type):
    monkeypatch.chdir(dummy_dist)
    subprocess.check_call(
        [
            sys.executable,
            "setup.py",
            "bdist_wheel",
            "-b",
            str(tmpdir),
            "--universal",
            f"--compression={option}",
        ]
    )
    with WheelFile("dist/dummy_dist-1.0-py2.py3-none-any.whl") as wf:
        filenames = set(wf.namelist())
        assert "dummy_dist-1.0.dist-info/RECORD" in filenames
        assert "dummy_dist-1.0.dist-info/METADATA" in filenames
        for zinfo in wf.filelist:
            assert zinfo.compress_type == compress_type


def test_wheelfile_line_endings(wheel_paths):
    for path in wheel_paths:
        with WheelFile(path) as wf:
            wheelfile = next(fn for fn in wf.filelist if fn.filename.endswith("WHEEL"))
            wheelfile_contents = wf.read(wheelfile)
            assert b"\r" not in wheelfile_contents


def test_unix_epoch_timestamps(dummy_dist, monkeypatch, tmpdir):
    monkeypatch.setenv("SOURCE_DATE_EPOCH", "0")
    monkeypatch.chdir(dummy_dist)
    subprocess.check_call(
        [
            sys.executable,
            "setup.py",
            "bdist_wheel",
            "-b",
            str(tmpdir),
            "--universal",
            "--build-number=2",
        ]
    )



"""
Tests for the bdist_wheel tag options (--python-tag, --universal, and
--plat-name)
"""

from __future__ import annotations

import subprocess
import sys

import pytest

SETUP_PY = """\
from setuptools import setup, Extension

setup(
    name="Test",
    version="1.0",
    author_email="author@example.com",
    py_modules=["test"],
    {ext_modules}
)
"""

EXT_MODULES = "ext_modules=[Extension('_test', sources=['test.c'])],"


@pytest.fixture
def temp_pkg(request, tmpdir):
    tmpdir.join("test.py").write('print("Hello, world")')

    ext = getattr(request, "param", [False, ""])
    if ext[0]:
        # if ext[1] is not '', it will write a bad header and fail to compile
        tmpdir.join("test.c").write("#include <std%sio.h>" % ext[1])
        setup_py = SETUP_PY.format(ext_modules=EXT_MODULES)
    else:
        setup_py = SETUP_PY.format(ext_modules="")

    tmpdir.join("setup.py").write(setup_py)
    if ext[0]:
        try:
            subprocess.check_call(
                [sys.executable, "setup.py", "build_ext"], cwd=str(tmpdir)
            )
        except subprocess.CalledProcessError:
            pytest.skip("Cannot compile C extensions")
    return tmpdir


@pytest.mark.parametrize("temp_pkg", [[True, "xxx"]], indirect=["temp_pkg"])
def test_nocompile_skips(temp_pkg):
    assert False  # noqa: B011 - should have skipped with a "Cannot compile" message


def test_default_tag(temp_pkg):
    subprocess.check_call(
        [sys.executable, "setup.py", "bdist_wheel"], cwd=str(temp_pkg)
    )
    dist_dir = temp_pkg.join("dist")
    assert dist_dir.check(dir=1)
    wheels = dist_dir.listdir()
    assert len(wheels) == 1
    assert wheels[0].basename == f"Test-1.0-py{sys.version_info[0]}-none-any.whl"
    assert wheels[0].ext == ".whl"


def test_build_number(temp_pkg):
    subprocess.check_call(
        [sys.executable, "setup.py", "bdist_wheel", "--build-number=1"],
        cwd=str(temp_pkg),
    )
    dist_dir = temp_pkg.join("dist")
    assert dist_dir.check(dir=1)
    wheels = dist_dir.listdir()
    assert len(wheels) == 1
    assert wheels[0].basename == f"Test-1.0-1-py{sys.version_info[0]}-none-any.whl"
    assert wheels[0].ext == ".whl"


def test_explicit_tag(temp_pkg):
    subprocess.check_call(
        [sys.executable, "setup.py", "bdist_wheel", "--python-tag=py32"],
        cwd=str(temp_pkg),
    )
    dist_dir = temp_pkg.join("dist")
    assert dist_dir.check(dir=1)
    wheels = dist_dir.listdir()
    assert len(wheels) == 1
    assert wheels[0].basename.startswith("Test-1.0-py32-")
    assert wheels[0].ext == ".whl"


def test_universal_tag(temp_pkg):
    subprocess.check_call(
        [sys.executable, "setup.py", "bdist_wheel", "--universal"], cwd=str(temp_pkg)
    )
    dist_dir = temp_pkg.join("dist")
    assert dist_dir.check(dir=1)
    wheels = dist_dir.listdir()
    assert len(wheels) == 1
    assert wheels[0].basename.startswith("Test-1.0-py2.py3-")
    assert wheels[0].ext == ".whl"


def test_universal_beats_explicit_tag(temp_pkg):
    subprocess.check_call(
        [sys.executable, "setup.py", "bdist_wheel", "--universal", "--python-tag=py32"],
        cwd=str(temp_pkg),
    )
    dist_dir = temp_pkg.join("dist")
    assert dist_dir.check(dir=1)
    wheels = dist_dir.listdir()
    assert len(wheels) == 1
    assert wheels[0].basename.startswith("Test-1.0-py2.py3-")
    assert wheels[0].ext == ".whl"


def test_universal_in_setup_cfg(temp_pkg):
    temp_pkg.join("setup.cfg").write("[bdist_wheel]\nuniversal=1")
    subprocess.check_call(
        [sys.executable, "setup.py", "bdist_wheel"], cwd=str(temp_pkg)
    )
    dist_dir = temp_pkg.join("dist")
    assert dist_dir.check(dir=1)
    wheels = dist_dir.listdir()
    assert len(wheels) == 1
    assert wheels[0].basename.startswith("Test-1.0-py2.py3-")
    assert wheels[0].ext == ".whl"


def test_pythontag_in_setup_cfg(temp_pkg):
    temp_pkg.join("setup.cfg").write("[bdist_wheel]\npython_tag=py32")
    subprocess.check_call(
        [sys.executable, "setup.py", "bdist_wheel"], cwd=str(temp_pkg)
    )
    dist_dir = temp_pkg.join("dist")
    assert dist_dir.check(dir=1)
    wheels = dist_dir.listdir()
    assert len(wheels) == 1
    assert wheels[0].basename.startswith("Test-1.0-py32-")
    assert wheels[0].ext == ".whl"


def test_legacy_wheel_section_in_setup_cfg(temp_pkg):
    temp_pkg.join("setup.cfg").write("[wheel]\nuniversal=1")
    subprocess.check_call(
        [sys.executable, "setup.py", "bdist_wheel"], cwd=str(temp_pkg)
    )
    dist_dir = temp_pkg.join("dist")
    assert dist_dir.check(dir=1)
    wheels = dist_dir.listdir()
    assert len(wheels) == 1
    assert wheels[0].basename.startswith("Test-1.0-py2.py3-")
    assert wheels[0].ext == ".whl"


def test_plat_name_purepy(temp_pkg):
    subprocess.check_call(
        [sys.executable, "setup.py", "bdist_wheel", "--plat-name=testplat.pure"],
        cwd=str(temp_pkg),
    )
    dist_dir = temp_pkg.join("dist")
    assert dist_dir.check(dir=1)
    wheels = dist_dir.listdir()
    assert len(wheels) == 1
    assert wheels[0].basename.endswith("-testplat_pure.whl")
    assert wheels[0].ext == ".whl"


@pytest.mark.parametrize("temp_pkg", [[True, ""]], indirect=["temp_pkg"])
def test_plat_name_ext(temp_pkg):
    subprocess.check_call(
        [sys.executable, "setup.py", "bdist_wheel", "--plat-name=testplat.arch"],
        cwd=str(temp_pkg),
    )

    dist_dir = temp_pkg.join("dist")
    assert dist_dir.check(dir=1)
    wheels = dist_dir.listdir()
    assert len(wheels) == 1
    assert wheels[0].basename.endswith("-testplat_arch.whl")
    assert wheels[0].ext == ".whl"


def test_plat_name_purepy_in_setupcfg(temp_pkg):
    temp_pkg.join("setup.cfg").write("[bdist_wheel]\nplat_name=testplat.pure")
    subprocess.check_call(
        [sys.executable, "setup.py", "bdist_wheel"], cwd=str(temp_pkg)
    )
    dist_dir = temp_pkg.join("dist")
    assert dist_dir.check(dir=1)
    wheels = dist_dir.listdir()
    assert len(wheels) == 1
    assert wheels[0].basename.endswith("-testplat_pure.whl")
    assert wheels[0].ext == ".whl"


@pytest.mark.parametrize("temp_pkg", [[True, ""]], indirect=["temp_pkg"])
def test_plat_name_ext_in_setupcfg(temp_pkg):
    temp_pkg.join("setup.cfg").write("[bdist_wheel]\nplat_name=testplat.arch")
    subprocess.check_call(
        [sys.executable, "setup.py", "bdist_wheel"], cwd=str(temp_pkg)
    )

    dist_dir = temp_pkg.join("dist")
    assert dist_dir.check(dir=1)
    wheels = dist_dir.listdir()
    assert len(wheels) == 1
    assert wheels[0].basename.endswith("-testplat_arch.whl")
    assert wheels[0].ext == ".whl"
